@@ -291,21 +291,18 @@ load_standard_apps (GsmManager *manager,
 
         autostart_dirs = gsm_util_get_autostart_dirs ();
 
-        append_default_apps (manager, default_session_key, autostart_dirs);
+        if (!failsafe) {
+                maybe_load_saved_session_apps (manager);
 
-        if (failsafe) {
-                goto out;
+                for (i = 0; autostart_dirs[i]; i++) {
+                        gsm_manager_add_autostart_apps_from_dir (manager,
+                                                                 autostart_dirs[i]);
+                }
         }
 
-        maybe_load_saved_session_apps (manager);
-
-        for (i = 0; autostart_dirs[i]; i++) {
-                gsm_manager_add_autostart_apps_from_dir (manager, autostart_dirs[i]);
-        }
-
- out:
         /* We do this at the end in case a saved session contains an
          * application that already provides one of the components. */
+        append_default_apps (manager, default_session_key, autostart_dirs);
         append_required_apps (manager);
 
         g_strfreev (autostart_dirs);
@@ -326,6 +323,7 @@ signal_cb (int      signo,
            gpointer data)
 {
         int ret;
+        GsmManager *manager;
 
         g_debug ("Got callback for signal %d", signo);
 
@@ -340,9 +338,12 @@ signal_cb (int      signo,
                 break;
         case SIGINT:
         case SIGTERM:
+                manager = (GsmManager *)data;
+                gsm_manager_logout (manager, GSM_MANAGER_LOGOUT_MODE_FORCE, NULL);
+
                 /* let the fatal signals interrupt us */
                 g_debug ("Caught signal %d, shutting down normally.", signo);
-                ret = FALSE;
+                ret = TRUE;
                 break;
         case SIGHUP:
                 g_debug ("Got HUP signal");
@@ -399,12 +400,12 @@ require_dbus_session (int      argc,
 
         /* +2 for our new arguments, +1 for NULL */
         new_argv = g_malloc (argc + 3 * sizeof (*argv));
-        
+
         new_argv[0] = "dbus-launch";
         new_argv[1] = "--exit-with-session";
-        new_argv[2] = argv[0];
-        for (i = 1; i < argc - 1; i++)
+        for (i = 0; i < argc; i++) {
                 new_argv[i + 2] = argv[i];
+	}
         new_argv[i + 2] = NULL;
         
         if (!execvp ("dbus-launch", new_argv)) {
@@ -473,14 +474,6 @@ main (int argc, char **argv)
         gdm_log_init ();
         gdm_log_set_debug (debug);
 
-        signal_handler = gdm_signal_handler_new ();
-        gdm_signal_handler_add_fatal (signal_handler);
-        gdm_signal_handler_add (signal_handler, SIGTERM, signal_cb, NULL);
-        gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, NULL);
-        gdm_signal_handler_add (signal_handler, SIGFPE, signal_cb, NULL);
-        gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
-        gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
-
         /* Set DISPLAY explicitly for all our children, in case --display
          * was specified on the command line.
          */
@@ -509,6 +502,13 @@ main (int argc, char **argv)
 
         manager = gsm_manager_new (client_store, failsafe);
 
+        signal_handler = gdm_signal_handler_new ();
+        gdm_signal_handler_add_fatal (signal_handler);
+        gdm_signal_handler_add (signal_handler, SIGFPE, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGTERM, signal_cb, manager);
+        gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, manager);
         gdm_signal_handler_set_fatal_func (signal_handler, shutdown_cb, manager);
 
         if (override_autostart_dirs != NULL) {
